module Main where

import Data.ByteString.UTF8 (fromString)
import Options.Applicative
import Redfa
import Version
import qualified Data.Vector as V
import System.IO.MMap

data DiffOptions =
  DiffOptions
    { diffRedfaOpt :: RedfaOption
    , diffFile1 :: String
    , diffFile2 :: String
    }
  deriving (Show)

diffOpts =
  DiffOptions <$> redfaOption <*> strArgument (metavar "FROMFILE") <*>
  strArgument (metavar "TOFILE")

main :: IO ()
main =
  let opts :: ParserInfo DiffOptions
      opts =
        info
          (diffOpts <**> versionOption "adiff" <**> helperOption)
          (fullDesc <>
           progDesc "Compare files by tokens and print out differences." <>
           header "adiff: arbitrary-token diff")
   in do o <- execParser opts
         redfa <- redfaPrepareRules (diffRedfaOpt o)
         data1 <- mmapFileByteString (diffFile1 o) Nothing
         data2 <- mmapFileByteString (diffFile2 o) Nothing
         toks1 <- V.fromList <$> redfaTokenize redfa data1
         toks2 <- V.fromList <$> redfaTokenize redfa data2
         print $ toks1
